@@ -14,7 +14,7 @@
  */
 
 #include <assert.h>
-#include <common/memory.h>
+#include <cgraph/alloc.h>
 #include <dotgen/dot.h>
 #include <limits.h>
 #include <math.h>
@@ -196,7 +196,7 @@ static void swap_bezier(bezier * old, bezier * new)
     int i, sz;
 
     sz = old->size;
-    list = N_GNEW(sz, pointf);
+    list = gv_calloc(sz, sizeof(pointf));
     lp = list;
     olp = old->list + (sz - 1);
     for (i = 0; i < sz; i++) {	/* reverse list of points */
@@ -221,7 +221,7 @@ static void swap_spline(splines * s)
     int i, sz;
 
     sz = s->size;
-    list = N_GNEW(sz, bezier);
+    list = gv_calloc(sz, sizeof(bezier));
     lp = list;
     olp = s->list + (sz - 1);
     for (i = 0; i < sz; i++) {	/* reverse and swap list of beziers */
@@ -290,7 +290,7 @@ setEdgeLabelPos (graph_t * g)
     for (n = GD_nlist(g); n; n = ND_next(n)) {
 	if (ND_node_type(n) == VIRTUAL) {
 	    if (ND_alg(n)) {   // label of non-adjacent flat edge
-		edge_t* fe = (edge_t*)ND_alg(n);
+		edge_t* fe = ND_alg(n);
 		l = ED_label(fe);
 		assert (l);
 		l->pos = ND_coord(n);
@@ -348,11 +348,11 @@ static void _dot_splines(graph_t * g, int normalize)
 
     mark_lowclusters(g);
     if (routesplinesinit()) return;
-    P = NEW(path);
+    P = gv_alloc(sizeof(path));
     /* FlatHeight = 2 * GD_nodesep(g); */
     sd.Splinesep = GD_nodesep(g) / 4;
     sd.Multisep = GD_nodesep(g);
-    edges = N_NEW(CHUNK, edge_t *);
+    edges = gv_calloc(CHUNK, sizeof(edge_t*));
 
     /* compute boundaries and list of splines */
     sd.LeftBound = sd.RightBound = 0;
@@ -372,7 +372,7 @@ static void _dot_splines(graph_t * g, int normalize)
 		 * the label.
 		 */
 	    if (ND_alg(n)) {
-		edge_t* fe = (edge_t*)ND_alg(n);
+		edge_t* fe = ND_alg(n);
 		assert (ED_label(fe));
 		ED_label(fe)->pos = ND_coord(n);
 		ED_label(fe)->set = true;
@@ -424,8 +424,8 @@ static void _dot_splines(graph_t * g, int normalize)
     qsort(edges, n_edges, sizeof(edges[0]), (qsort_cmpf)edgecmp);
 
     /* FIXME: just how many boxes can there be? */
-    P->boxes = N_NEW(n_nodes + 20 * 2 * NSUB, boxf);
-    sd.Rank_box = N_NEW(i, boxf);
+    P->boxes = gv_calloc(n_nodes + 20 * 2 * NSUB, sizeof(boxf));
+    sd.Rank_box = gv_calloc(i, sizeof(boxf));
 
     if (et == EDGETYPE_LINE) {
     /* place regular edge labels */
@@ -479,7 +479,7 @@ static void _dot_splines(graph_t * g, int normalize)
 	    if (cnt == 1)
 		edgelist = &e0;
 	    else
-		edgelist = N_NEW(cnt, edge_t*);
+		edgelist = gv_calloc(cnt, sizeof(edge_t*));
 	    edgelist[0] = getmainedge((edges+ind)[0]);
 	    for (ii = 1; ii < cnt; ii++)
 		edgelist[ii] = (edges+ind)[ii];
@@ -488,7 +488,8 @@ static void _dot_splines(graph_t * g, int normalize)
 		free (edgelist);
 	}
 	else if (agtail(e0) == aghead(e0)) {
-	    int b, sizey, r;
+	    int b, r;
+	    double sizey;
 	    n = agtail(e0);
 	    r = ND_rank(n);
 	    if (r == GD_maxrank(g)) {
@@ -501,8 +502,8 @@ static void _dot_splines(graph_t * g, int normalize)
 		sizey = ND_coord(n).y - ND_coord(GD_rank(g)[r+1].v[0]).y;
 	    }
 	    else {
-		int upy = ND_coord(GD_rank(g)[r-1].v[0]).y - ND_coord(n).y;
-		int dwny = ND_coord(n).y - ND_coord(GD_rank(g)[r+1].v[0]).y;
+		double upy = ND_coord(GD_rank(g)[r-1].v[0]).y - ND_coord(n).y;
+		double dwny = ND_coord(n).y - ND_coord(GD_rank(g)[r+1].v[0]).y;
 		sizey = MIN(upy, dwny);
 	    }
 	    makeSelfEdge(edges, ind, cnt, sd.Multisep, sizey / 2, &sinfo);
@@ -654,12 +655,16 @@ static int edgecmp(edge_t** ptr0, edge_t** ptr1)
 
     fwdedgea.out.base.data = (Agrec_t*)&fwdedgeai;
     fwdedgeb.out.base.data = (Agrec_t*)&fwdedgebi;
-    e0 = (edge_t *) * ptr0;
-    e1 = (edge_t *) * ptr1;
+    e0 = *ptr0;
+    e1 = *ptr1;
     et0 = ED_tree_index(e0) & EDGETYPEMASK;
     et1 = ED_tree_index(e1) & EDGETYPEMASK;
-    if (et0 != et1)
-	return (et1 - et0);
+    if (et0 < et1) {
+	return 1;
+    }
+    if (et0 > et1) {
+	return -1;
+    }
 
     le0 = getmainedge(e0);
     le1 = getmainedge(e1);
@@ -668,19 +673,31 @@ static int edgecmp(edge_t** ptr0, edge_t** ptr1)
     t1 = ND_rank(agtail(le1)) - ND_rank(aghead(le1));
     v0 = abs((int)t0);   /* ugly, but explicit as to how we avoid equality tests on fp numbers */
     v1 = abs((int)t1);
-    if (v0 != v1)
-	return (v0 - v1);
+    if (v0 < v1) {
+	return -1;
+    }
+    if (v0 > v1) {
+	return 1;
+    }
 
     t0 = ND_coord(agtail(le0)).x - ND_coord(aghead(le0)).x;
     t1 = ND_coord(agtail(le1)).x - ND_coord(aghead(le1)).x;
     v0 = abs((int)t0);
     v1 = abs((int)t1);
-    if (v0 != v1)
-	return (v0 - v1);
+    if (v0 < v1) {
+	return -1;
+    }
+    if (v0 > v1) {
+	return 1;
+    }
 
     /* This provides a cheap test for edges having the same set of endpoints.  */
-    if (AGSEQ(le0) != AGSEQ(le1))
-	return (AGSEQ(le0) - AGSEQ(le1));
+    if (AGSEQ(le0) < AGSEQ(le1)) {
+	return -1;
+    }
+    if (AGSEQ(le0) > AGSEQ(le1)) {
+	return 1;
+    }
 
     ea = (ED_tail_port(e0).defined || ED_head_port(e0).defined) ? e0 : le0;
     if (ED_tree_index(ea) & BWDEDGE) {
@@ -699,13 +716,29 @@ static int edgecmp(edge_t** ptr0, edge_t** ptr1)
 
     et0 = ED_tree_index(e0) & GRAPHTYPEMASK;
     et1 = ED_tree_index(e1) & GRAPHTYPEMASK;
-    if (et0 != et1)
-	return (et0 - et1);
+    if (et0 < et1) {
+	return -1;
+    }
+    if (et0 > et1) {
+	return 1;
+    }
 
-    if (et0 == FLATEDGE && ED_label(e0) != ED_label(e1))
-	return (int) (ED_label(e0) - ED_label(e1));
+    if (et0 == FLATEDGE) {
+	if (ED_label(e0) < ED_label(e1)) {
+	    return -1;
+	}
+	if (ED_label(e0) > ED_label(e1)) {
+	    return 1;
+	}
+    }
 
-    return (AGSEQ(e0) - AGSEQ(e1));
+    if (AGSEQ(e0) < AGSEQ(e1)) {
+	return -1;
+    }
+    if (AGSEQ(e0) > AGSEQ(e1)) {
+	return 1;
+    }
+    return 0;
 }
 
 /* cloneGraph:
@@ -858,7 +891,7 @@ cloneGraph (graph_t* g, attr_state_t* attr_state)
 	auxg = agopen ("auxg",Agundirected, NULL);
     agbindrec(auxg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
     agattr(auxg, AGRAPH, "rank", "");
-    GD_drawing(auxg) = NEW(layout_t);
+    GD_drawing(auxg) = gv_alloc(sizeof(layout_t));
     GD_drawing(auxg)->quantum = GD_drawing(g)->quantum; 
     GD_drawing(auxg)->dpi = GD_drawing(g)->dpi;
 
@@ -942,21 +975,19 @@ cleanupCloneGraph (graph_t* g, attr_state_t* attr_state)
 }
 
 /* cloneNode:
- * If flipped is true, original graph has rankdir=LR or RL.
- * In this case, records change shape, so we wrap a record node's
- * label in "{...}" to prevent this.
+ * If original graph has rankdir=LR or RL, records change shape,
+ * so we wrap a record node's label in "{...}" to prevent this.
  */
-static node_t*
-cloneNode (graph_t* g, node_t* orign, int flipped)
-{
+static node_t *cloneNode(graph_t *g, node_t *orign) {
     node_t* n = agnode(g, agnameof(orign),1);
     agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
     agcopyattr (orign, n);
     if (shapeOf(orign) == SH_RECORD) {
-	int lbllen = strlen(ND_label(orign)->text);
-        char* buf = N_GNEW(lbllen+3,char);
+        size_t lbllen = strlen(ND_label(orign)->text);
+        char* buf = gv_calloc(lbllen + 3, sizeof(char));
         sprintf (buf, "{%s}", ND_label(orign)->text);
 	agset (n, "label", buf);
+        free(buf);
     }
 
     return n;
@@ -996,11 +1027,10 @@ transformf (pointf p, pointf del, int flip)
  */
 static int edgelblcmpfn(edge_t** ptr0, edge_t** ptr1)
 {
-    edge_t *e0, *e1;
     pointf sz0, sz1;
 
-    e0 = (edge_t *) * ptr0;
-    e1 = (edge_t *) * ptr1;
+    edge_t *e0 = *ptr0;
+    edge_t *e1 = *ptr1;
 
     if (ED_label(e0)) {
 	if (ED_label(e1)) {
@@ -1042,7 +1072,7 @@ makeSimpleFlatLabels (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt,
     double uminx, umaxx;
     double lminx=0.0, lmaxx=0.0;
 
-    edge_t** earray = N_NEW(cnt, edge_t*);
+    edge_t** earray = gv_calloc(cnt, sizeof(edge_t*));
 
     for (i = 0; i < cnt; i++) {
 	earray[i] = edges[ind + i];
@@ -1239,7 +1269,7 @@ makeSimpleFlat (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt, int et
  * more straightforward and laborious fashion. 
  */
 static void
-make_flat_adj_edges(graph_t* g, path* P, edge_t** edges, int ind, int cnt, edge_t* e0,
+make_flat_adj_edges(graph_t* g, edge_t** edges, int ind, int cnt, edge_t* e0,
                     int et)
 {
     node_t* n;
@@ -1250,7 +1280,8 @@ make_flat_adj_edges(graph_t* g, path* P, edge_t** edges, int ind, int cnt, edge_
     graph_t* subg;
     node_t *auxt, *auxh;
     edge_t* auxe;
-    int     i, j, midx, midy, leftx, rightx;
+    int     i, j;
+    double midx, midy, leftx, rightx;
     pointf   del;
     edge_t* hvye = NULL;
     attr_state_t* attrs;
@@ -1285,7 +1316,7 @@ make_flat_adj_edges(graph_t* g, path* P, edge_t** edges, int ind, int cnt, edge_
 	return;
     }
 
-    attrs = NEW(attr_state_t);
+    attrs = gv_alloc(sizeof(attr_state_t));
     auxg = cloneGraph (g, attrs);
     subg = agsubg (auxg, "xxx",1);
     agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
@@ -1298,8 +1329,8 @@ make_flat_adj_edges(graph_t* g, path* P, edge_t** edges, int ind, int cnt, edge_
         tn = hn;
         hn = n;
     }
-    auxt = cloneNode(subg, tn, GD_flip(g)); 
-    auxh = cloneNode(auxg, hn, GD_flip(g)); 
+    auxt = cloneNode(subg, tn);
+    auxh = cloneNode(auxg, hn);
     for (i = 0; i < cnt; i++) {
 	e = edges[ind + i];
 	for (; ED_edge_type(e) != NORMAL; e = ED_to_orig(e));
@@ -1438,7 +1469,7 @@ make_flat_labeled_edge(graph_t* g, spline_info_t* sp, path* P, edge_t* e, int et
     bool ps_needs_free = false;
     pathend_t tend, hend;
     boxf lb;
-    int i, pn, ydelta;
+    int i, pn;
     edge_t *f;
     pointf points[7];
 
@@ -1468,8 +1499,8 @@ make_flat_labeled_edge(graph_t* g, spline_info_t* sp, path* P, edge_t* e, int et
 	lb.LL.x = ND_coord(ln).x - ND_lw(ln);
 	lb.UR.x = ND_coord(ln).x + ND_rw(ln);
 	lb.UR.y = ND_coord(ln).y + ND_ht(ln)/2;
-	ydelta = ND_coord(ln).y - GD_rank(g)[ND_rank(tn)].ht1 -
-		ND_coord(tn).y + GD_rank(g)[ND_rank(tn)].ht2;
+	double ydelta = ND_coord(ln).y - GD_rank(g)[ND_rank(tn)].ht1 -
+	                ND_coord(tn).y + GD_rank(g)[ND_rank(tn)].ht2;
 	ydelta /= 6;
 	lb.LL.y = lb.UR.y - MAX(5,ydelta);
 
@@ -1617,7 +1648,7 @@ make_flat_edge(graph_t* g, spline_info_t* sp, path * P, edge_t ** edges, int ind
      * so check them all.
      */
     if (isAdjacent) {
-	make_flat_adj_edges (g, P, edges, ind, cnt, e, et);
+	make_flat_adj_edges(g, edges, ind, cnt, e, et);
 	return;
     }
     if (ED_label(e)) {  /* edges with labels aren't multi-edges */
@@ -1818,8 +1849,8 @@ make_regular_edge(graph_t* g, spline_info_t* sp, path * P, edge_t ** edges, int
     fwdedge.out.base.data = (Agrec_t*)&fwdedgei;
 
     if (!pointfs) {
-	pointfs = N_GNEW(NUMPTS, pointf);
-   	pointfs2 = N_GNEW(NUMPTS, pointf);
+	pointfs = gv_calloc(NUMPTS, sizeof(pointf));
+   	pointfs2 = gv_calloc(NUMPTS, sizeof(pointf));
 	numpts = NUMPTS;
 	numpts2 = NUMPTS;
     }
