@@ -10,43 +10,32 @@
 
 #include <sfdpgen/Multilevel.h>
 #include <sfdpgen/PriorityQueue.h>
-#include <common/memory.h>
-#include <common/logic.h>
 #include <assert.h>
+#include <cgraph/alloc.h>
 #include <common/arith.h>
 #include <stddef.h>
+#include <stdbool.h>
 
-Multilevel_control Multilevel_control_new(int scheme, int mode){
-  Multilevel_control ctrl;
-
-  ctrl = GNEW(struct Multilevel_control_struct);
+Multilevel_control Multilevel_control_new(void) {
+  Multilevel_control ctrl = gv_alloc(sizeof(struct Multilevel_control_struct));
   ctrl->minsize = 4;
   ctrl->min_coarsen_factor = 0.75;
   ctrl->maxlevel = 1<<30;
   ctrl->randomize = TRUE;
-  /* now set in spring_electrical_control_new(), as well as by command line argument -c
-    ctrl->coarsen_scheme = COARSEN_INDEPENDENT_EDGE_SET_HEAVEST_CLUSTER_PERNODE_LEAVES_FIRST;
-    ctrl->coarsen_scheme = COARSEN_INDEPENDENT_VERTEX_SET_RS;
-    ctrl->coarsen_scheme = COARSEN_INDEPENDENT_EDGE_SET_HEAVEST_EDGE_PERNODE;
-    ctrl->coarsen_scheme = COARSEN_HYBRID;
-    ctrl->coarsen_scheme = COARSEN_INDEPENDENT_EDGE_SET_HEAVEST_EDGE_PERNODE_SUPERNODES_FIRST;
-    ctrl->coarsen_mode = COARSEN_MODE_FORCEFUL; or COARSEN_MODE_GENTLE;
-  */
 
-  ctrl->coarsen_scheme = scheme;
-  ctrl->coarsen_mode = mode;
+  ctrl->coarsen_scheme = COARSEN_INDEPENDENT_EDGE_SET_HEAVEST_EDGE_PERNODE_SUPERNODES_FIRST;
+  ctrl->coarsen_mode = COARSEN_MODE_FORCEFUL;
   return ctrl;
 }
 
 void Multilevel_control_delete(Multilevel_control ctrl){
-  FREE(ctrl);
+  free(ctrl);
 }
 
-static Multilevel Multilevel_init(SparseMatrix A, SparseMatrix D, real *node_weights){
-  Multilevel grid;
+static Multilevel Multilevel_init(SparseMatrix A, SparseMatrix D, double *node_weights){
   if (!A) return NULL;
   assert(A->m == A->n);
-  grid = GNEW(struct Multilevel_struct);
+  Multilevel grid = gv_alloc(sizeof(struct Multilevel_struct));
   grid->level = 0;
   grid->n = A->n;
   grid->A = A;
@@ -75,14 +64,14 @@ void Multilevel_delete(Multilevel grid){
   }
   SparseMatrix_delete(grid->P);
   SparseMatrix_delete(grid->R);
-  if (grid->node_weights && grid->level > 0) FREE(grid->node_weights);
+  if (grid->node_weights && grid->level > 0) free(grid->node_weights);
   Multilevel_delete(grid->next);
-  FREE(grid);
+  free(grid);
 }
 
 static void maximal_independent_vertex_set(SparseMatrix A, int randomize, int **vset, int *nvset, int *nzc){
   int i, ii, j, *ia, *ja, m, n, *p = NULL;
-  NOTUSED(n);
+  (void)n;
   assert(A);
   assert(SparseMatrix_known_strucural_symmetric(A));
   ia = A->ia;
@@ -90,7 +79,7 @@ static void maximal_independent_vertex_set(SparseMatrix A, int randomize, int **
   m = A->m;
   n = A->n;
   assert(n == m);
-  *vset = N_GNEW(m,int);
+  *vset = gv_calloc(m, sizeof(int));
   for (i = 0; i < m; i++) (*vset)[i] = MAX_IND_VTX_SET_U;
   *nvset = 0;
   *nzc = 0;
@@ -119,7 +108,7 @@ static void maximal_independent_vertex_set(SparseMatrix A, int randomize, int **
 	}
       }
     }
-    FREE(p);
+    free(p);
   }
   (*nzc) += *nvset;
 }
@@ -135,8 +124,8 @@ static void maximal_independent_vertex_set_RS(SparseMatrix A, int randomize, int
    */
   int i, jj, ii, *p = NULL, j, k, *ia, *ja, m, n, gain, removed, nf = 0;
   PriorityQueue q;
-  NOTUSED(removed);
-  NOTUSED(n);
+  (void)removed;
+  (void)n;
   assert(A);
   assert(SparseMatrix_known_strucural_symmetric(A));
 
@@ -145,7 +134,7 @@ static void maximal_independent_vertex_set_RS(SparseMatrix A, int randomize, int
   m = A->m;
   n = A->n;
   assert(n == m);
-  *vset = N_GNEW(m,int);
+  *vset = gv_calloc(m, sizeof(int));
   for (i = 0; i < m; i++) {
     (*vset)[i] = MAX_IND_VTX_SET_U;
   }
@@ -163,7 +152,7 @@ static void maximal_independent_vertex_set_RS(SparseMatrix A, int randomize, int
       i = p[ii];
       PriorityQueue_push(q, i, ia[i+1] - ia[i]);
     }
-    FREE(p);
+    free(p);
   }
 
   while (PriorityQueue_pop(q, &i, &gain)){
@@ -208,7 +197,7 @@ static void maximal_independent_edge_set(SparseMatrix A, int randomize, int **ma
   m = A->m;
   n = A->n;
   assert(n == m);
-  *matching = N_GNEW(m,int);
+  *matching = gv_calloc(m, sizeof(int));
   for (i = 0; i < m; i++) (*matching)[i] = i;
   *nmatch = n;
 
@@ -236,7 +225,7 @@ static void maximal_independent_edge_set(SparseMatrix A, int randomize, int **ma
 	}
       }
     }
-    FREE(p);
+    free(p);
   }
 }
 
@@ -244,7 +233,7 @@ static void maximal_independent_edge_set(SparseMatrix A, int randomize, int **ma
 
 static void maximal_independent_edge_set_heavest_edge_pernode(SparseMatrix A, int randomize, int **matching, int *nmatch){
   int i, ii, j, *ia, *ja, m, n, *p = NULL;
-  real *a, amax = 0;
+  double *a, amax = 0;
   int first = TRUE, jamax = 0;
 
   assert(A);
@@ -254,14 +243,14 @@ static void maximal_independent_edge_set_heavest_edge_pernode(SparseMatrix A, in
   m = A->m;
   n = A->n;
   assert(n == m);
-  *matching = N_GNEW(m,int);
+  *matching = gv_calloc(m, sizeof(int));
   for (i = 0; i < m; i++) (*matching)[i] = i;
   *nmatch = n;
 
-  assert(SparseMatrix_is_symmetric(A, FALSE));
+  assert(SparseMatrix_is_symmetric(A, false));
   assert(A->type == MATRIX_TYPE_REAL);
 
-  a = (real*) A->a;
+  a = A->a;
   if (!randomize){
     for (i = 0; i < m; i++){
       first = TRUE;
@@ -313,7 +302,7 @@ static void maximal_independent_edge_set_heavest_edge_pernode(SparseMatrix A, in
 	  (*nmatch)--;
       }
     }
-    FREE(p);
+    free(p);
   }
 }
 
@@ -325,11 +314,11 @@ static void maximal_independent_edge_set_heavest_edge_pernode(SparseMatrix A, in
 
 static void maximal_independent_edge_set_heavest_edge_pernode_leaves_first(SparseMatrix A, int randomize, int **cluster, int **clusterp, int *ncluster){
   int i, ii, j, *ia, *ja, m, n, *p = NULL, q;
-  NOTUSED(n);
-  real *a, amax = 0;
+  (void)n;
+  double *a, amax = 0;
   int first = TRUE, jamax = 0;
   int *matched, nz, ncmax = 0, nz0, nzz,k ;
-  enum {UNMATCHED = -2, MATCHED = -1};
+  enum {MATCHED = -1};
 
   assert(A);
   assert(SparseMatrix_known_strucural_symmetric(A));
@@ -338,19 +327,19 @@ static void maximal_independent_edge_set_heavest_edge_pernode_leaves_first(Spars
   m = A->m;
   n = A->n;
   assert(n == m);
-  *cluster = N_GNEW(m,int);
-  *clusterp = N_GNEW((m+1),int);
-  matched = N_GNEW(m,int);
+  *cluster = gv_calloc(m, sizeof(int));
+  *clusterp = gv_calloc(m + 1, sizeof(int));
+  matched = gv_calloc(m, sizeof(int));
 
   for (i = 0; i < m; i++) matched[i] = i;
 
-  assert(SparseMatrix_is_symmetric(A, FALSE));
+  assert(SparseMatrix_is_symmetric(A, false));
   assert(A->type == MATRIX_TYPE_REAL);
 
   *ncluster = 0;
   (*clusterp)[0] = 0;
   nz = 0;
-  a = (real*) A->a;
+  a = A->a;
   if (!randomize){
     for (i = 0; i < m; i++){
       if (matched[i] == MATCHED || node_degree(i) != 1) continue;
@@ -411,7 +400,6 @@ static void maximal_independent_edge_set_heavest_edge_pernode_leaves_first(Spars
       }
     }
 
-    /* dan yi dian, wu ban */
     for (i = 0; i < m; i++){
       if (matched[i] == i){
 	(*cluster)[nz++] = i;
@@ -483,7 +471,6 @@ static void maximal_independent_edge_set_heavest_edge_pernode_leaves_first(Spars
       }
     }
 
-    /* dan yi dian, wu ban */
     for (i = 0; i < m; i++){
       if (matched[i] == i){
 	(*cluster)[nz++] = i;
@@ -491,21 +478,21 @@ static void maximal_independent_edge_set_heavest_edge_pernode_leaves_first(Spars
       }
     }
 
-    FREE(p);
+    free(p);
   }
 
-  FREE(matched);
+  free(matched);
 }
 
 
 
 static void maximal_independent_edge_set_heavest_edge_pernode_supernodes_first(SparseMatrix A, int randomize, int **cluster, int **clusterp, int *ncluster){
   int i, ii, j, *ia, *ja, m, n, *p = NULL;
-  NOTUSED(n);
-  real *a, amax = 0;
+  (void)n;
+  double *a, amax = 0;
   int first = TRUE, jamax = 0;
   int *matched, nz, nz0;
-  enum {UNMATCHED = -2, MATCHED = -1};
+  enum {MATCHED = -1};
   int  nsuper, *super = NULL, *superp = NULL;
 
   assert(A);
@@ -515,13 +502,13 @@ static void maximal_independent_edge_set_heavest_edge_pernode_supernodes_first(S
   m = A->m;
   n = A->n;
   assert(n == m);
-  *cluster = N_GNEW(m,int);
-  *clusterp = N_GNEW((m+1),int);
-  matched = N_GNEW(m,int);
+  *cluster = gv_calloc(m, sizeof(int));
+  *clusterp = gv_calloc(m + 1, sizeof(int));
+  matched = gv_calloc(m, sizeof(int));
 
   for (i = 0; i < m; i++) matched[i] = i;
 
-  assert(SparseMatrix_is_symmetric(A, FALSE));
+  assert(SparseMatrix_is_symmetric(A, false));
   assert(A->type == MATRIX_TYPE_REAL);
 
   SparseMatrix_decompose_to_supervariables(A, &nsuper, &super, &superp);
@@ -529,7 +516,7 @@ static void maximal_independent_edge_set_heavest_edge_pernode_supernodes_first(S
   *ncluster = 0;
   (*clusterp)[0] = 0;
   nz = 0;
-  a = (real*) A->a;
+  a = A->a;
 
   for (i = 0; i < nsuper; i++){
     if (superp[i+1] - superp[i] <= 1) continue;
@@ -573,7 +560,6 @@ static void maximal_independent_edge_set_heavest_edge_pernode_supernodes_first(S
       }
     }
 
-    /* dan yi dian, wu ban */
     for (i = 0; i < m; i++){
       if (matched[i] == i){
 	(*cluster)[nz++] = i;
@@ -612,45 +598,43 @@ static void maximal_independent_edge_set_heavest_edge_pernode_supernodes_first(S
       }
     }
 
-    /* dan yi dian, wu ban */
     for (i = 0; i < m; i++){
       if (matched[i] == i){
 	(*cluster)[nz++] = i;
 	(*clusterp)[++(*ncluster)] = nz;
       }
     }
-    FREE(p);
+    free(p);
 
   }
 
-  FREE(super);
+  free(super);
 
-  FREE(superp);
+  free(superp);
 
-  FREE(matched);
+  free(matched);
 }
 
 static int scomp(const void *s1, const void *s2){
-  const real *ss1, *ss2;
-  ss1 = (const real*) s1;
-  ss2 = (const real*) s2;
+  const double *ss1 = s1;
+  const double *ss2 = s2;
 
-  if ((ss1)[1] > (ss2)[1]){
+  if (ss1[1] > ss2[1]){
     return -1;
-  } else if ((ss1)[1] < (ss2)[1]){
+  } else if (ss1[1] < ss2[1]){
     return 1;
   }
   return 0;
 }
 
 static void maximal_independent_edge_set_heavest_cluster_pernode_leaves_first(SparseMatrix A, int csize, 
-									      int randomize, int **cluster, int **clusterp, int *ncluster){
+									      int **cluster, int **clusterp, int *ncluster){
   int i, ii, j, *ia, *ja, m, n, *p = NULL, q, iv;
-  NOTUSED(n);
-  real *a;
+  (void)n;
+  double *a;
   int *matched, nz,  nz0, nzz,k, nv;
-  enum {UNMATCHED = -2, MATCHED = -1};
-  real *vlist;
+  enum {MATCHED = -1};
+  double *vlist;
 
   assert(A);
   assert(SparseMatrix_known_strucural_symmetric(A));
@@ -659,20 +643,20 @@ static void maximal_independent_edge_set_heavest_cluster_pernode_leaves_first(Sp
   m = A->m;
   n = A->n;
   assert(n == m);
-  *cluster = N_GNEW(m,int);
-  *clusterp = N_GNEW((m+1),int);
-  matched = N_GNEW(m,int);
-  vlist = N_GNEW(2*m,real);
+  *cluster = gv_calloc(m, sizeof(int));
+  *clusterp = gv_calloc(m + 1, sizeof(int));
+  matched = gv_calloc(m, sizeof(int));
+  vlist = gv_calloc(2 * m, sizeof(double));
 
   for (i = 0; i < m; i++) matched[i] = i;
 
-  assert(SparseMatrix_is_symmetric(A, FALSE));
+  assert(SparseMatrix_is_symmetric(A, false));
   assert(A->type == MATRIX_TYPE_REAL);
 
   *ncluster = 0;
   (*clusterp)[0] = 0;
   nz = 0;
-  a = (real*) A->a;
+  a = A->a;
 
   p = random_permutation(m);
   for (ii = 0; ii < m; ii++){
@@ -716,7 +700,7 @@ static void maximal_independent_edge_set_heavest_cluster_pernode_leaves_first(Sp
       }
     }
     if (nv > 0){
-      qsort(vlist, nv, sizeof(real)*2, scomp);
+      qsort(vlist, nv, sizeof(double)*2, scomp);
       for (j = 0; j < MIN(csize - 1, nv); j++){
 	iv = (int) vlist[2*j];
 	matched[iv] = MATCHED;
@@ -728,21 +712,20 @@ static void maximal_independent_edge_set_heavest_cluster_pernode_leaves_first(Sp
     }
   }
   
-  /* dan yi dian, wu ban */
   for (i = 0; i < m; i++){
     if (matched[i] == i){
       (*cluster)[nz++] = i;
       (*clusterp)[++(*ncluster)] = nz;
     }
   }
-  FREE(p);
+  free(p);
 
 
-  FREE(matched);
+  free(matched);
 }
 static void maximal_independent_edge_set_heavest_edge_pernode_scaled(SparseMatrix A, int randomize, int **matching, int *nmatch){
   int i, ii, j, *ia, *ja, m, n, *p = NULL;
-  real *a, amax = 0;
+  double *a, amax = 0;
   int first = TRUE, jamax = 0;
 
   assert(A);
@@ -752,14 +735,14 @@ static void maximal_independent_edge_set_heavest_edge_pernode_scaled(SparseMatri
   m = A->m;
   n = A->n;
   assert(n == m);
-  *matching = N_GNEW(m,int);
+  *matching = gv_calloc(m, sizeof(int));
   for (i = 0; i < m; i++) (*matching)[i] = i;
   *nmatch = n;
 
-  assert(SparseMatrix_is_symmetric(A, FALSE));
+  assert(SparseMatrix_is_symmetric(A, false));
   assert(A->type == MATRIX_TYPE_REAL);
 
-  a = (real*) A->a;
+  a = A->a;
   if (!randomize){
     for (i = 0; i < m; i++){
       first = TRUE;
@@ -811,33 +794,16 @@ static void maximal_independent_edge_set_heavest_edge_pernode_scaled(SparseMatri
 	  (*nmatch)--;
       }
     }
-    FREE(p);
+    free(p);
   }
 }
 
-static SparseMatrix DistanceMatrix_restrict_cluster(int ncluster, int *clusterp, int *cluster, SparseMatrix P, SparseMatrix R, SparseMatrix D){
-  return NULL;
-}
-
-static SparseMatrix DistanceMatrix_restrict_matching(int *matching, SparseMatrix D){
-  if (!D) return NULL;
-  assert(0);/* not yet implemented! */
-  return NULL;
-}
-
-static SparseMatrix DistanceMatrix_restrict_filtering(int *mask, int is_C, int is_F, SparseMatrix D){
-  /* max independent vtx set based coarsening. Coarsen nodes has mask >= is_C. Fine nodes == is_F. */
-  if (!D) return NULL;
-  assert(0);/* not yet implemented! */
-  return NULL;
-}
-
 static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, SparseMatrix D, SparseMatrix *cD,
-					real *node_wgt, real **cnode_wgt,
+					double *node_wgt, double **cnode_wgt,
 					SparseMatrix *P, SparseMatrix *R, Multilevel_control ctrl, int *coarsen_scheme_used){
   int *matching = NULL, nmatch = 0, nc, nzc, n, i;
   int *irn = NULL, *jcn = NULL, *ia = NULL, *ja = NULL;
-  real *val = NULL;
+  double *val = NULL;
   SparseMatrix B = NULL;
   int *vset = NULL, nvset, ncov, j;
   int *cluster=NULL, *clusterp=NULL, ncluster;
@@ -906,7 +872,7 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
     } else if (ctrl->coarsen_scheme == COARSEN_INDEPENDENT_EDGE_SET_HEAVEST_EDGE_PERNODE_SUPERNODES_FIRST) {
       maximal_independent_edge_set_heavest_edge_pernode_supernodes_first(A, ctrl->randomize, &cluster, &clusterp, &ncluster);
     } else {
-      maximal_independent_edge_set_heavest_cluster_pernode_leaves_first(A, 4, ctrl->randomize, &cluster, &clusterp, &ncluster);
+      maximal_independent_edge_set_heavest_cluster_pernode_leaves_first(A, 4, &cluster, &clusterp, &ncluster);
     }
     assert(ncluster <= n);
     nc = ncluster;
@@ -917,9 +883,9 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
 #endif
       goto RETURN;
     }
-    irn = N_GNEW(n,int);
-    jcn = N_GNEW(n,int);
-    val = N_GNEW(n,real);
+    irn = gv_calloc(n, sizeof(int));
+    jcn = gv_calloc(n, sizeof(int));
+    val = gv_calloc(n, sizeof(double));
     nzc = 0; 
     for (i = 0; i < ncluster; i++){
       for (j = clusterp[i]; j < clusterp[i+1]; j++){
@@ -930,10 +896,11 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
      }
     }
     assert(nzc == n);
-    *P = SparseMatrix_from_coordinate_arrays(nzc, n, nc, irn, jcn, (void *) val, MATRIX_TYPE_REAL, sizeof(real));
+    *P = SparseMatrix_from_coordinate_arrays(nzc, n, nc, irn, jcn, val,
+                                             MATRIX_TYPE_REAL, sizeof(double));
     *R = SparseMatrix_transpose(*P);
 
-    *cD = DistanceMatrix_restrict_cluster(ncluster, clusterp, cluster, *P, *R, D);
+    *cD = NULL;
 
     *cA = SparseMatrix_multiply3(*R, A, *P); 
 
@@ -944,7 +911,7 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
       */
     if (!*cA) goto RETURN;
 
-    SparseMatrix_multiply_vector(*R, node_wgt, cnode_wgt, FALSE);
+    SparseMatrix_multiply_vector(*R, node_wgt, cnode_wgt);
     *R = SparseMatrix_divide_row_by_degree(*R);
     SparseMatrix_set_symmetric(*cA);
     SparseMatrix_set_pattern_symmetric(*cA);
@@ -969,9 +936,9 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
 #endif
       goto RETURN;
     }
-    irn = N_GNEW(n,int);
-    jcn = N_GNEW(n,int);
-    val = N_GNEW(n,real);
+    irn = gv_calloc(n, sizeof(int));
+    jcn = gv_calloc(n, sizeof(int));
+    val = gv_calloc(n, sizeof(double));
     nzc = 0; nc = 0;
     for (i = 0; i < n; i++){
       if (matching[i] >= 0){
@@ -994,7 +961,8 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
     }
     assert(nc == nmatch);
     assert(nzc == n);
-    *P = SparseMatrix_from_coordinate_arrays(nzc, n, nc, irn, jcn, (void *) val, MATRIX_TYPE_REAL, sizeof(real));
+    *P = SparseMatrix_from_coordinate_arrays(nzc, n, nc, irn, jcn, val,
+                                             MATRIX_TYPE_REAL, sizeof(double));
     *R = SparseMatrix_transpose(*P);
     *cA = SparseMatrix_multiply3(*R, A, *P); 
     /*
@@ -1003,15 +971,14 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
       *cA = SparseMatrix_multiply(B, *P); 
       */
     if (!*cA) goto RETURN;
-    SparseMatrix_multiply_vector(*R, node_wgt, cnode_wgt, FALSE);
+    SparseMatrix_multiply_vector(*R, node_wgt, cnode_wgt);
     *R = SparseMatrix_divide_row_by_degree(*R);
     SparseMatrix_set_symmetric(*cA);
     SparseMatrix_set_pattern_symmetric(*cA);
     *cA = SparseMatrix_remove_diagonal(*cA);
 
 
-    *cD = DistanceMatrix_restrict_matching(matching, D);
-    *cD=NULL;
+    *cD = NULL;
 
     break;
   case COARSEN_INDEPENDENT_VERTEX_SET:
@@ -1031,9 +998,9 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
 #endif
       goto RETURN;
     }
-    irn = N_GNEW(nzc,int);
-    jcn = N_GNEW(nzc,int);
-    val = N_GNEW(nzc,real);
+    irn = gv_calloc(nzc, sizeof(int));
+    jcn = gv_calloc(nzc, sizeof(int));
+    val = gv_calloc(nzc, sizeof(double));
     nzc = 0; 
     for (i = 0; i < n; i++){
       if (vset[i] == MAX_IND_VTX_SET_F){
@@ -1059,36 +1026,40 @@ static void Multilevel_coarsen_internal(SparseMatrix A, SparseMatrix *cA, Sparse
       }
     }
 
-    *P = SparseMatrix_from_coordinate_arrays(nzc, n, nc, irn, jcn, (void *) val, MATRIX_TYPE_REAL, sizeof(real));
+    *P = SparseMatrix_from_coordinate_arrays(nzc, n, nc, irn, jcn, val,
+                                             MATRIX_TYPE_REAL, sizeof(double));
     *R = SparseMatrix_transpose(*P);
     *cA = SparseMatrix_multiply3(*R, A, *P); 
     if (!*cA) goto RETURN;
-    SparseMatrix_multiply_vector(*R, node_wgt, cnode_wgt, FALSE);
+    SparseMatrix_multiply_vector(*R, node_wgt, cnode_wgt);
     SparseMatrix_set_symmetric(*cA);
     SparseMatrix_set_pattern_symmetric(*cA);
     *cA = SparseMatrix_remove_diagonal(*cA);
 
-    *cD = DistanceMatrix_restrict_filtering(vset, MAX_IND_VTX_SET_C, MAX_IND_VTX_SET_F, D);
+    // TODO: restrict filtering, the operation that would happen here, has not
+    // been implemented
+    assert(!D);
+    *cD = NULL;
     break;
   default:
     goto RETURN;
   }
  RETURN:
-  if (matching) FREE(matching);
-  if (vset) FREE(vset);
-  if (irn) FREE(irn);
-  if (jcn) FREE(jcn);
-  if (val) FREE(val);
+  free(matching);
+  free(vset);
+  free(irn);
+  free(jcn);
+  free(val);
   if (B) SparseMatrix_delete(B);
 
-  if(cluster) FREE(cluster);
-  if(clusterp) FREE(clusterp);
+  free(cluster);
+  free(clusterp);
 }
 
-void Multilevel_coarsen(SparseMatrix A, SparseMatrix *cA, SparseMatrix D, SparseMatrix *cD, real *node_wgt, real **cnode_wgt,
+void Multilevel_coarsen(SparseMatrix A, SparseMatrix *cA, SparseMatrix D, SparseMatrix *cD, double *node_wgt, double **cnode_wgt,
 			       SparseMatrix *P, SparseMatrix *R, Multilevel_control ctrl, int *coarsen_scheme_used){
   SparseMatrix cA0 = A,  cD0 = NULL, P0 = NULL, R0 = NULL, M;
-  real *cnode_wgt0 = NULL;
+  double *cnode_wgt0 = NULL;
   int nc = 0, n;
   
   *P = NULL; *R = NULL; *cA = NULL; *cnode_wgt = NULL, *cD = NULL;
@@ -1123,7 +1094,7 @@ void Multilevel_coarsen(SparseMatrix A, SparseMatrix *cA, SparseMatrix D, Sparse
     if (*cD) SparseMatrix_delete(*cD);
     *cD = cD0;
 
-    if (*cnode_wgt) FREE(*cnode_wgt);
+    if (*cnode_wgt) free(*cnode_wgt);
     *cnode_wgt = cnode_wgt0;
     A = cA0;
     D = cD0;
@@ -1140,7 +1111,7 @@ void print_padding(int n){
 static Multilevel Multilevel_establish(Multilevel grid, Multilevel_control ctrl){
   Multilevel cgrid;
   int coarsen_scheme_used;
-  real *cnode_weights = NULL;
+  double *cnode_weights = NULL;
   SparseMatrix P, R, A, cA, D, cD;
 
 #ifdef DEBUG_PRINT
@@ -1178,18 +1149,18 @@ static Multilevel Multilevel_establish(Multilevel grid, Multilevel_control ctrl)
   
 }
 
-Multilevel Multilevel_new(SparseMatrix A0, SparseMatrix D0, real *node_weights, Multilevel_control ctrl){
+Multilevel Multilevel_new(SparseMatrix A0, SparseMatrix D0, Multilevel_control ctrl){
   /* A: the weighting matrix. D: the distance matrix, could be NULL. If not null, the two matrices must have the same sparsity pattern */
   Multilevel grid;
   SparseMatrix A = A0, D = D0;
 
-  if (!SparseMatrix_is_symmetric(A, FALSE) || A->type != MATRIX_TYPE_REAL){
+  if (!SparseMatrix_is_symmetric(A, false) || A->type != MATRIX_TYPE_REAL){
     A = SparseMatrix_get_real_adjacency_matrix_symmetrized(A);
   }
-  if (D && (!SparseMatrix_is_symmetric(D, FALSE) || D->type != MATRIX_TYPE_REAL)){
+  if (D && (!SparseMatrix_is_symmetric(D, false) || D->type != MATRIX_TYPE_REAL)){
     D = SparseMatrix_symmetrize_nodiag(D);
   }
-  grid = Multilevel_init(A, D, node_weights);
+  grid = Multilevel_init(A, D, NULL);
   grid = Multilevel_establish(grid, ctrl);
   if (A != A0) grid->delete_top_level_A = TRUE;/* be sure to clean up later */
   return grid;
