@@ -27,14 +27,14 @@
 #include <gvc/gvplugin_render.h>
 #include <gvc/gvplugin_device.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
+#include <cgraph/startswith.h>
 #include <cgraph/unreachable.h>
 #include <common/utils.h>
 #include <gvc/gvc.h>
 #include <gvc/gvio.h>
 #include <gvc/gvcint.h>
 
-#include <common/memory.h>
-
 typedef enum {
 	FORMAT_JSON,
 	FORMAT_JSON0,
@@ -58,7 +58,9 @@ typedef struct {
 #define ED_gid(n) (((gvid_t*)aggetrec(n, ID, FALSE))->id) 
 #define GD_gid(n) (((gvid_t*)aggetrec(n, ID, FALSE))->id) 
 
-#define IS_CLUSTER(s) (!strncmp(agnameof(s), "cluster", 7))
+static bool IS_CLUSTER(Agraph_t *s) {
+  return startswith(agnameof(s), "cluster");
+}
 
 static void json_begin_graph(GVJ_t *job)
 {
@@ -158,12 +160,11 @@ static void set_attrwf(Agraph_t * g, bool toplevel, bool value)
 
 static void write_polyline (GVJ_t * job, xdot_polyline* polyline)
 {
-    int i;
-    int cnt = polyline->cnt;
+    const size_t cnt = polyline->cnt;
     xdot_point* pts = polyline->pts;
 
     gvprintf(job, "\"points\": [");
-    for (i = 0; i < cnt; i++) {
+    for (size_t i = 0; i < cnt; i++) {
 	if (i > 0) gvprintf(job, ",");
 	gvprintf(job, "[%.03f,%.03f]", pts[i].x, pts[i].y);
     }
@@ -308,7 +309,6 @@ static void write_xdot (xdot_op * op, GVJ_t * job, state_t* sp)
 static void write_xdots (char * val, GVJ_t * job, state_t* sp)
 {
     xdot* cmds;
-    int i;
 
     if (!val || *val == '\0') return;
 
@@ -321,7 +321,7 @@ static void write_xdots (char * val, GVJ_t * job, state_t* sp)
     gvputs(job, "\n");
     indent(job, sp->Level++);
     gvputs(job, "[\n");
-    for (i = 0; i < cmds->cnt; i++) {
+    for (size_t i = 0; i < cmds->cnt; i++) {
 	if (i > 0)
 	    gvputs(job, ",\n");
 	write_xdot (cmds->ops+i, job, sp);
@@ -333,12 +333,10 @@ static void write_xdots (char * val, GVJ_t * job, state_t* sp)
     freeXDot(cmds);
 }
 
-static int isXDot (char* name)
-{
-  return *name++ == '_' &&
-          (streq(name,"draw_") || streq(name,"ldraw_") ||
-          streq(name,"hdraw_") || streq(name,"tdraw_") ||
-          streq(name,"hldraw_") || streq(name,"tldraw_"));
+static bool isXDot(const char* name) {
+  return streq(name, "_draw_") || streq(name, "_ldraw_") ||
+         streq(name, "_hdraw_") || streq(name, "_tdraw_") ||
+         streq(name, "_hldraw_") || streq(name, "_tldraw_");
 }
 
 static void write_attrs(Agobj_t * obj, GVJ_t * job, state_t* sp)
@@ -362,8 +360,7 @@ static void write_attrs(Agobj_t * obj, GVJ_t * job, state_t* sp)
     }
 }
 
-static void write_hdr(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
-{
+static void write_hdr(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
     char *name;
 
     name = agnameof(g);
@@ -379,25 +376,27 @@ static void write_hdr(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
     }
 }
 
-static void write_graph(Agraph_t * g, GVJ_t * job, int top, state_t* sp);
+static void write_graph(Agraph_t *g, GVJ_t *job, bool top, state_t *sp);
 
 static void write_subg(Agraph_t * g, GVJ_t * job, state_t* sp)
 {
     Agraph_t* sg;
 
-    write_graph (g, job, FALSE, sp);
+    write_graph (g, job, false, sp);
     for (sg = agfstsubg(g); sg; sg = agnxtsubg(sg)) {
 	gvputs(job, ",\n");
 	write_subg(sg, job, sp);
     }
 }
 
-static int write_subgs(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
-{
+/// write out subgraphs
+///
+/// \return True if the given graph has any subgraphs
+static bool write_subgs(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
     Agraph_t* sg;
 
     sg = agfstsubg(g);
-    if (!sg) return 0;
+    if (!sg) return false;
    
     gvputs(job, ",\n");
     indent(job, sp->Level++);
@@ -423,7 +422,7 @@ static int write_subgs(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
 	gvputs(job, "]");
     }
 
-    return 1;
+    return true;
 }
 
 static int agseqasc(Agedge_t **lhs, Agedge_t **rhs)
@@ -442,8 +441,7 @@ static int agseqasc(Agedge_t **lhs, Agedge_t **rhs)
     }
 }
 
-static void write_edge(Agedge_t * e, GVJ_t * job, int top, state_t* sp)
-{
+static void write_edge(Agedge_t *e, GVJ_t *job, bool top, state_t *sp) {
     if (top) {
 	indent(job, sp->Level++);
 	gvputs(job, "{\n");
@@ -464,8 +462,7 @@ static void write_edge(Agedge_t * e, GVJ_t * job, int top, state_t* sp)
     }
 }
 
-static int write_edges(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
-{
+static int write_edges(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
     size_t count = 0;
 
     for (Agnode_t *np = agfstnode(g); np; np = agnxtnode(g, np)) {
@@ -478,7 +475,7 @@ static int write_edges(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
         return 0;
     }
 
-    Agedge_t **edges = gcalloc(count, sizeof(Agedge_t *));
+    Agedge_t **edges = gv_calloc(count, sizeof(Agedge_t *));
 
     size_t i = 0;
     for (Agnode_t *np = agfstnode(g); np; np = agnxtnode(g, np)) {
@@ -514,8 +511,7 @@ static int write_edges(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
     return 1;
 }
 
-static void write_node(Agnode_t * n, GVJ_t * job, int top, state_t* sp)
-{
+static void write_node(Agnode_t *n, GVJ_t *job, bool top, state_t *sp) {
     if (top) {
 	indent(job, sp->Level++);
 	gvputs(job, "{\n");
@@ -534,12 +530,18 @@ static void write_node(Agnode_t * n, GVJ_t * job, int top, state_t* sp)
     }
 }
 
-static int write_nodes(Agraph_t * g, GVJ_t * job, int top, int has_subgs, state_t* sp)
-{
-    Agnode_t* n;
+static int write_nodes(Agraph_t *g, GVJ_t *job, bool top, bool has_subgs, state_t *sp) {
 
-    n = agfstnode(g);
-    if (!n) {
+    // is every subcomponent of this graph a cluster?
+    bool only_clusters = true;
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
+	if (!IS_CLUST_NODE(n)) {
+	    only_clusters = false;
+	    break;
+	}
+    }
+
+    if (only_clusters) {
 	if (has_subgs && top) {
 	    sp->Level--;
 	    gvputs(job, "\n");
@@ -561,7 +563,7 @@ static int write_nodes(Agraph_t * g, GVJ_t * job, int top, int has_subgs, state_
 	indent(job, sp->Level);
     }
     const char *separator = "";
-    for (; n; n = agnxtnode(g, n)) {
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (IS_CLUST_NODE(n)) continue;
 	gvputs(job, separator);
 	write_node (n, job, top, sp);
@@ -616,7 +618,7 @@ static void insert (Dt_t* map, char* name, int v)
 	    agerr(AGWARN, "Duplicate cluster name \"%s\"\n", name);
 	return;
     }
-    ip = calloc(1, sizeof(intm));
+    ip = gv_alloc(sizeof(intm));
     ip->id = strdup(name);
     ip->v = v;
     dtinsert (map, ip);
@@ -638,14 +640,12 @@ static int label_subgs(Agraph_t* g, int lbl, Dt_t* map)
 }
 
 
-static void write_graph(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
-{
+static void write_graph(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
     Agnode_t* np; 
     Agedge_t* ep; 
     int ncnt = 0;
     int ecnt = 0;
     int sgcnt = 0;
-    int has_subgs;
     Dt_t* map;
 
     if (top) {
@@ -681,7 +681,7 @@ static void write_graph(Agraph_t * g, GVJ_t * job, int top, state_t* sp)
 	indent(job, sp->Level);
 	gvprintf(job, "\"_gvid\": %d", GD_gid(g));
     }
-    has_subgs = write_subgs(g, job, top, sp);
+    bool has_subgs = write_subgs(g, job, top, sp);
     write_nodes (g, job, top, has_subgs, sp);
     write_edges (g, job, top, sp);
     gvputs(job, "\n");
@@ -714,7 +714,7 @@ static void json_end_graph(GVJ_t *job)
     sp.Level = 0;
     sp.isLatin = GD_charset(g) == CHAR_LATIN1;
     sp.doXDot = job->render.id == FORMAT_JSON || job->render.id == FORMAT_XDOT_JSON;
-    write_graph(g, job, TRUE, &sp);
+    write_graph(g, job, true, &sp);
 }
 
 gvrender_engine_t json_engine = {
