@@ -109,6 +109,42 @@ std::string SVG::to_dot_color(const std::string &color, double opacity) {
   return rgb_to_hex(color, opacity);
 }
 
+void SVG::SVGElement::add_bbox() {
+  const auto bbox = SVGElement::bbox();
+  add_rect(bbox, "green");
+}
+
+void SVG::SVGElement::add_rect(SVGRect rect, const std::string color) {
+  SVG::SVGElement element{SVG::SVGElementType::Rect};
+  element.attributes.x = rect.x;
+  element.attributes.y = rect.y;
+  element.attributes.width = rect.width;
+  element.attributes.height = rect.height;
+  element.attributes.stroke_width = 0.1;
+  element.attributes.stroke = color;
+  element.attributes.fill = "none";
+  children.push_back(element);
+}
+
+void SVG::SVGElement::add_outline_bbox() {
+  const auto bbox = SVGElement::outline_bbox();
+  add_rect(bbox, "blue");
+}
+
+void SVG::SVGElement::add_outline_overlap_bbox(SVG::SVGElement other,
+                                               const double tolerance) {
+  const auto bbox = outline_bbox();
+  const auto other_bbox = other.outline_bbox();
+  const auto overlap_bbox = bbox.intersection(other_bbox);
+  if (overlap_bbox.width <= 0 || overlap_bbox.height <= 0) {
+    return;
+  }
+  const auto within_tolerance =
+      overlap_bbox.width <= tolerance && overlap_bbox.height <= tolerance;
+  const auto color = within_tolerance ? "yellow" : "red";
+  add_rect(overlap_bbox, color);
+}
+
 SVG::SVGRect SVG::SVGElement::bbox(bool throw_if_bbox_not_defined) {
   if (!m_bbox.has_value()) {
     // negative width and height bbox that will be immediately replaced by the
@@ -278,6 +314,7 @@ SVG::SVGRect SVG::SVGElement::outline_bbox(bool throw_if_bbox_not_defined) {
         }
         break;
       }
+      // FIXME: this throws for the `curve` and `icurve` edge arrows
       throw std::runtime_error(
           "paths other than straight vertical, straight horizontal or the "
           "cylinder special case are currently not supported");
@@ -380,6 +417,22 @@ SVG::SVGRect SVG::SVGElement::outline_bbox(bool throw_if_bbox_not_defined) {
   return *m_bbox;
 }
 
+SVG::SVGElement &SVG::SVGElement::find_child(const SVG::SVGElementType type,
+                                             std::size_t index) {
+  std::size_t i = 0;
+  for (auto &child : children) {
+    if (child.type == type) {
+      if (i == index) {
+        return child;
+      }
+      ++i;
+    }
+  }
+  throw std::runtime_error(
+      fmt::format("SVG element only has {} \"{}\" children, index {} not found",
+                  i, tag(type), index));
+}
+
 SVG::SVGRect SVG::SVGElement::text_bbox() const {
   assert(type == SVG::SVGElementType::Text && "Not a 'text' element");
 
@@ -611,6 +664,15 @@ void SVG::SVGElement::to_string_impl(std::string &output,
     append_attribute(attributes_str, stroke_opacity_attribute_to_string());
     append_attribute(attributes_str, points_attribute_to_string());
     break;
+  case SVG::SVGElementType::Rect:
+    attributes_str +=
+        fmt::format(R"(x="{}" y="{}" width="{}" height="{}")", attributes.x,
+                    attributes.y, attributes.width, attributes.height);
+    append_attribute(attributes_str, fill_attribute_to_string());
+    append_attribute(attributes_str, stroke_attribute_to_string());
+    append_attribute(attributes_str, stroke_width_attribute_to_string());
+    append_attribute(attributes_str, stroke_opacity_attribute_to_string());
+    break;
   case SVG::SVGElementType::Svg:
     attributes_str += fmt::format(
         R"(width="{}pt" height="{}pt")"
@@ -738,7 +800,7 @@ SVG::SVGElement::miter_point(SVG::SVGPoint segment_start,
    *
    * The distance between P and P1 and between P and P2 is stroke-width / 2.
    *
-   * NOTE: This method only implements the 'miter' join and does not fallback to
+   * NOTE: This method only implements the 'miter' join, but falls back to
    * 'bevel' when stroke-miterlimit is exceeded.
    */
 
@@ -771,6 +833,27 @@ SVG::SVGElement::miter_point(SVG::SVGPoint segment_start,
   const auto beta_rev = beta - std::numbers::pi;
   const auto theta = beta_rev - alpha;
 
+  const auto miter_limit = 4.0;
+  const auto miter_length = stroke_width / std::sin(std::abs(theta) / 2.0);
+
+  if (miter_length > miter_limit * stroke_width) {
+    // fall back to bevel
+    const auto sinBeta = dyB / hypotB;
+    const auto sinBetaMinusPi = -sinBeta;
+    const auto cosBetaMinusPi = -cosBeta;
+    const SVG::SVGPoint P2 = {P.x + stroke_width / 2.0 * sinBetaMinusPi,
+                              P.y + stroke_width / 2.0 * cosBetaMinusPi};
+
+    // the bevel is the triangle formed from the three points P, P1 and P2 so
+    // a good enough approximation of the miter point in this case is the
+    // crossing of P-P3 with P1-P2 which is the same as the midpoint between
+    // P1 and P2
+    const SVG::SVGPoint Pbevel = {(P1.x + P2.x) / 2, (P1.y + P2.y) / 2};
+
+    // SVG has inverted y axis so invert the returned y value
+    return {Pbevel.x, -Pbevel.y};
+  }
+
   // length between P1 and P3 (and between P2 and P3)
   const auto l = stroke_width / 2.0 / std::tan(theta / 2.0);
 
